# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Anaconda Cloud login dialog."""

from .account_dialogs import *  # noqa: F403
from .anaconda_dialogs import *  # noqa: F403
from .commercial_dialogs import *  # noqa: F403
from .enterprise_dialogs import *  # noqa: F403
from .nucleus_dialogs import *  # noqa: F403
from .team_dialogs import *  # noqa: F403


# --- Local testing
# -----------------------------------------------------------------------------
def local_test():  # pragma: no cover
    """Run local test."""
    from anaconda_navigator.static.fonts import load_fonts
    from anaconda_navigator.utils.qthelpers import qapplication
    app = qapplication(test_time=3)
    load_fonts(app)

    # ENTERPRISE EDITION TESTING
    widgets_iter = (EnterpriseRepoSetDomainPage, EnterpriseRepoLoginPage, NoticePage)  # noqa: F405
    for widget in widgets_iter:
        app.w = widget()
        exit_code = app.w.exec()
        print(exit_code)
        if not exit_code:
            break


if __name__ == '__main__':  # pragma: no cover
    local_test()
